@@ -11,9 +11,7 @@ from itertools import chain
 from pathlib import Path
 from shutil import which
 from types import TracebackType
-
-# TODO: replace Optinal with | None
-from typing import Optional
+from typing import TypeVar
 
 from diopter.utils import run_cmd
 
@@ -68,6 +66,9 @@ class Language(Enum):
                 return ".cpp"
 
 
+ProgramType = TypeVar("ProgramType", bound="SourceProgram")
+
+
 @dataclass(frozen=True, kw_only=True)
 class SourceProgram:
     """A C or C++ source program together with flags, includes and macro definitions.
@@ -135,11 +136,42 @@ class SourceProgram:
 
         return self.language.to_suffix()
 
-    def with_code(self, new_code: str) -> SourceProgram:
+    def get_modified_code(self) -> str:
+        """Returns `self.code` potentially modified to be used in `CompilationSetting`.
+
+        Subclasses of `SourceProgram` can override this method.
+
+        Returns:
+            str:
+                the (modified) `self.code`
+        """
+        return self.code
+
+    def with_preprocessed_code(
+        self: ProgramType, preprocessed_code: str
+    ) -> ProgramType:
+        """Returns a new program with its code replaced with `preprocessed_code`
+
+        `CompilerSetting.preprocess_program` calls this. It can be overriden by
+        subclasses.
+
+        Returns:
+            ProgramType:
+                the new program
+        """
+        return replace(
+            self,
+            code=preprocessed_code,
+            defined_macros=tuple(),
+            include_paths=tuple(),
+            system_include_paths=tuple(),
+        )
+
+    def with_code(self: ProgramType, new_code: str) -> ProgramType:
         """Returns a new program with its code replaced with new_code
 
         Returns:
-            SourceProgram:
+            ProgramType:
                 the new program
         """
         return replace(self, code=new_code)
@@ -319,9 +351,9 @@ class TemporaryFile:
 
     def __exit__(
         self,
-        exc_type: Optional[type[BaseException]],
-        exc_value: Optional[BaseException],
-        exc_traceback: Optional[TracebackType],
+        exc_type: type[BaseException] | None,
+        exc_value: BaseException | None,
+        exc_traceback: TracebackType | None,
     ) -> None:
         if self.path and Path(self.path).exists():
             os.remove(self.path)
@@ -413,7 +445,7 @@ class CompilationSetting:
 
     def get_compilation_cmd(
         self,
-        program: tuple[SourceProgram, Path],
+        program: tuple[ProgramType, Path],
         output: CompilationOutput,
         include_language_flags: bool = True,
     ) -> list[str]:
@@ -421,8 +453,8 @@ class CompilationSetting:
         input programs and the output.
 
         Args:
-            program (tuple[SourceProgram, Path]):
-                flags from the SourcePrograms are extracted and
+            program (tuple[ProgramType, Path]):
+                flags from the program are extracted and
                 the corresponding path are included in the output
             output (CompilationOutput):
                 the output path and potentially additional flags
@@ -460,14 +492,14 @@ class CompilationSetting:
 
     def compile_program(
         self,
-        program: SourceProgram,
+        program: ProgramType,
         output: CompilationOutput,
         additional_flags: tuple[str, ...] = tuple(),
-        timeout: Optional[int] = None,
+        timeout: int | None = None,
     ) -> CompilationInfo:
 
         with TemporaryFile(
-            contents=program.code, suffix=program.get_file_suffix()
+            contents=program.get_modified_code(), suffix=program.get_file_suffix()
         ) as code_file:
             cmd = self.get_compilation_cmd((program, code_file), output, True) + list(
                 additional_flags
@@ -488,14 +520,14 @@ class CompilationSetting:
             )
 
     def get_llvm_ir_from_program(
-        self, program: SourceProgram, timeout: Optional[int] = None
+        self, program: ProgramType, timeout: int | None = None
     ) -> str:
         """Extracts LLVM-IR from the program.
 
         This only works with LLVM compilers.
 
         Args:
-           program (SourceProgram): input program
+           program (ProgramType): input program
            timeout (int | None): timeout in seconds for the compilation command
         Returns:
             str:
@@ -505,14 +537,14 @@ class CompilationSetting:
 
     def get_asm_from_program(
         self,
-        program: SourceProgram,
+        program: ProgramType,
         additional_flags: tuple[str, ...] = (),
-        timeout: Optional[int] = None,
+        timeout: int | None = None,
     ) -> str:
         """Extracts assembly code from the program.
 
         Args:
-           program (SourceProgram): input program
+           program (ProgramType): input program
            additional_flags (tuple[str, ...]): additional flags used for the compilation
            timeout (int | None): timeout in seconds for the compilation command
 
@@ -532,15 +564,15 @@ class CompilationSetting:
 
     def compile_program_to_object(
         self,
-        program: SourceProgram,
+        program: ProgramType,
         object_file: Path,
         additional_flags: tuple[str, ...] = tuple(),
-        timeout: Optional[int] = None,
+        timeout: int | None = None,
     ) -> CompilationInfo:
         """Compiles program to object file
 
         Args:
-           program (SourceProgram): input program
+           program (ProgramType): input program
            object_file (Path): path to the output
            additional_flags (tuple[str, ...]): additional flags used for the compilation
            timeout (int | None): timeout in seconds for the compilation command
@@ -559,15 +591,15 @@ class CompilationSetting:
 
     def compile_program_to_executable(
         self,
-        program: SourceProgram,
+        program: ProgramType,
         executable_path: Path,
         additional_flags: tuple[str, ...] = tuple(),
-        timeout: Optional[int] = None,
+        timeout: int | None = None,
     ) -> CompilationInfo:
         """Compiles program to object file
 
         Args:
-           program (SourceProgram): input program
+           program (ProgramType): input program
            executable_file (Path): path to the output
            additional_flags (tuple[str, ...]): additional flags used for the compilation
            timeout (int | None): timeout in seconds for the compilation command
@@ -587,15 +619,15 @@ class CompilationSetting:
 
     def preprocess_program(
         self,
-        program: SourceProgram,
+        program: ProgramType,
         make_compiler_agnostic: bool = False,
         additional_flags: tuple[str, ...] = tuple(),
-        timeout: Optional[int] = None,
-    ) -> SourceProgram:
+        timeout: int | None = None,
+    ) -> ProgramType:
         """Preprocesses the program
 
         Args:
-           program (SourceProgram): input program
+           program (ProgramType): input program
            additional_flags (tuple[str, ...]): additional flags used for the compilation
            make_compiler_agnostic (bool):
                if true will try to remove certain constructs (e.g., attributes)
@@ -629,13 +661,7 @@ class CompilationSetting:
                 r"typedef [^;]*_Float\d+x?;", r"", preprocessed_source
             )
 
-        return replace(
-            program,
-            code=preprocessed_source,
-            defined_macros=tuple(),
-            include_paths=tuple(),
-            system_include_paths=tuple(),
-        )
+        return program.with_preprocessed_code(preprocessed_source)
 
 
 def find_standard_include_paths(
@@ -731,7 +757,7 @@ class ClangTool:
         program: SourceProgram,
         tool_flags: list[str],
         mode: ClangToolMode,
-        timeout: Optional[int] = None,
+        timeout: int | None = None,
     ) -> ClangToolResult:
         """Run the clang tool on the input program
 
@@ -752,7 +778,7 @@ class ClangTool:
         """
 
         with TemporaryFile(
-            contents=program.code, suffix=program.get_file_suffix()
+            contents=program.get_modified_code(), suffix=program.get_file_suffix()
         ) as tf:
             match program.language:
                 case Language.C:
@@ -811,7 +837,7 @@ class CComp:
     # TODO: timeout_s: int = 8
 
     @staticmethod
-    def get_system_ccomp() -> Optional[CComp]:
+    def get_system_ccomp() -> CComp | None:
         """Returns:
         CComp:
             the system's ccomp
@@ -823,7 +849,7 @@ class CComp:
         return CComp(exe=Path(ccomp).resolve(strict=True))
 
     def check_program(
-        self, program: SourceProgram, timeout: Optional[int] = None, debug: bool = False
+        self, program: SourceProgram, timeout: int | None = None, debug: bool = False
     ) -> bool:
         """Checks the input program for errors using ccomp's interpreter mode.
 
@@ -839,7 +865,7 @@ class CComp:
         assert program.language == Language.C
 
         # ccomp doesn't like these
-        code = re.sub(r"__asm__ [^\)]*\)", r"", program.code)
+        code = re.sub(r"__asm__ [^\)]*\)", r"", program.get_modified_code())
 
         with TemporaryFile(contents=code, suffix=".c") as tf:
             cmd = (
@@ -911,7 +937,7 @@ def parse_compilation_setting_from_string(
         s (str): compilation command string to be parsed.
 
     Returns:
-        tuple[CompilationSetting, tuple[str, ...], list[str], CompilationOutput]:
+        tuple[CompilationSetting, list[str], CompilationOutput]:
             - CompilationSetting
             - source files provided
             - specified compilation output file and kind
